<?php


/*
  ----------------------------------------------------------------------
  GLPI - Gestionnaire Libre de Parc Informatique
  Copyright (C) 2003-2008 by the INDEPNET Development Team.
  
  http://indepnet.net/   http://glpi-project.org/
  ----------------------------------------------------------------------
  
  LICENSE
  
  This file is part of GLPI.
  
  GLPI is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  GLPI is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with GLPI; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  ------------------------------------------------------------------------
 */

// ----------------------------------------------------------------------
// Original Author of file: CAILLAUD Xavier
// Purpose of file:
// ----------------------------------------------------------------------

if (!defined('GLPI_ROOT')) {
	die("Sorry. You can't access directly to this file");
}

function plugin_massocsimport_isScriptLocked() {
	return file_exists(PLUGIN_MASSOCSIMPORT_LOCKFILE);
}
function plugin_massocsimport_setScriptLock() {
	$fp = fopen(PLUGIN_MASSOCSIMPORT_LOCKFILE, "w+");
	fclose($fp);
}

function plugin_massocsimport_removeScriptLock() {
	if (file_exists(PLUGIN_MASSOCSIMPORT_LOCKFILE))
		unlink(PLUGIN_MASSOCSIMPORT_LOCKFILE);
}

function plugin_massocsimport_getAllOcsServers() {
	global $DB, $LANG;

	$servers[-1] = $LANG["massocsimport"]["config"][22];
	$sql = "SELECT ID,name FROM glpi_ocs_config";
	$result = $DB->query($sql);
	while ($conf = $DB->fetch_array($result))
		$servers[$conf["ID"]] = $conf["name"];

	return $servers;
}

function plugin_massocsimport_install() {
	$DB = new DB;

	if (plugin_massocsimport_needUpgrade()) {
		//Upgrade process if needed
		plugin_massocsimport_upgrade11to12();
		plugin_massocsimport_upgrade121to13();
	} else {
		$query = "CREATE TABLE IF NOT EXISTS `glpi_plugin_massocsimport_not_imported` (
						  `ID` INT( 11 ) NOT NULL  auto_increment,
						  `ocs_id` INT( 11 ) NOT NULL ,
						  `ocs_server_id` INT( 11 ) NOT NULL ,
						  `deviceid` VARCHAR( 255 ) NOT NULL ,
						  `useragent` VARCHAR( 255 ) NOT NULL ,
						  `tag` VARCHAR( 255 ) NOT NULL ,
						  `name` VARCHAR( 255 ) NOT NULL ,
						  `ipaddr` VARCHAR( 255 ) NOT NULL ,
						  `domain` VARCHAR( 255 ) NOT NULL ,
						  `last_inventory` DATETIME ,
						  PRIMARY KEY ( `ID` ),
						  UNIQUE KEY `ocs_id` (`ocs_server_id`,`ocs_id`)
						) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci ";

		$DB->query($query) or die($DB->error());

		$query = "CREATE TABLE IF NOT EXISTS `glpi_plugin_massocsimport` (
						  `ID` int(11) NOT NULL AUTO_INCREMENT,
						  `thread_id` int(4) NOT NULL default '0',
						  `start_time` datetime NOT NULL default '0000-00-00 00:00:00',
						  `end_time` datetime NOT NULL default '0000-00-00 00:00:00',
						  `status` int(4) NOT NULL default '0',
						  `error_msg` text NOT NULL,
						  `imported_machines_number` int(11) NOT NULL default '0',
						  `synchronized_machines_number` int(11) NOT NULL default '0',
						  `failed_rules_machines_number` int(11) NOT NULL default '0',
						  `linked_machines_number` int(11) NOT NULL default '0',
						  `ocs_server_id` int(4) NOT NULL default '1',
						  `total_number_machines` int(11) NOT NULL default '0',
						  `process_id` bigint(20) NOT NULL,
						  `noupdate_machines_number` int(11) NOT NULL default '0',
						  PRIMARY KEY  (`ID`),
						  KEY `end_time` (`end_time`)
						) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci ";
		$DB->query($query) or die($DB->error());

		$query = "CREATE TABLE IF NOT EXISTS `glpi_plugin_massocsimport_servers` (
							`ID` int(11) NOT NULL AUTO_INCREMENT,
							`ocs_server_id` int(11) NOT NULL,
							`max_ocs_id` int(11) DEFAULT NULL,
							`max_glpi_date` datetime DEFAULT NULL,
							PRIMARY KEY (`ID`),
							UNIQUE KEY `ocs_server_id` (`ocs_server_id`)
							) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci ";

		$DB->query($query) or die($DB->error());

		$query = "CREATE TABLE IF NOT EXISTS `glpi_plugin_massocsimport_config` (
							`ID` int(11) NOT NULL,
							`thread_log_frequency` int(4) NOT NULL default '10',
							`display_empty` int(1) NOT NULL default '1',
							`delete_frequency` int(4) NOT NULL default '0',
							`import_limit` int(11) NOT NULL default '0',
							`default_ocs_server` int(11) NOT NULL default '-1',
							`delay_refresh` varchar(4) NOT NULL default '0',
							`comments` text,
							PRIMARY KEY  (`ID`)
							) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci ";
		$DB->query($query) or die($DB->error());

		$query = "INSERT INTO `glpi_plugin_massocsimport_config` (`ID`, `thread_log_frequency`," .
		" `display_empty`, `delete_frequency`, `import_limit`,`default_ocs_server` ) " .
		" VALUES (1, 2, 1, 24, 0,-1);";

		$DB->query($query) or die($DB->error());

		$query = "CREATE TABLE IF NOT EXISTS `glpi_plugin_massocsimport_details` (
				  `ID` int(11) NOT NULL auto_increment,
				  `process_id` bigint(10) NOT NULL default '0',
				  `thread_id` int(4) NOT NULL default '0',
				  `ocs_id` int(11) NOT NULL default '0',
				  `glpi_id` int(11) NOT NULL default '0',
				  `action` int(11) NOT NULL default '0',
				  `process_time` datetime DEFAULT NULL,
				  `ocs_server_id` int(4) NOT NULL default '1',
				  PRIMARY KEY  (`ID`),
				  KEY `end_time` (`process_time`)
				) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci";
		$DB->query($query) or die($DB->error());

	$query = "INSERT INTO `glpi_display` (`type`, `num`, `rank`, `FK_users`) VALUES
					(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 2, 1, 0),
					(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 3, 2, 0),
					(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 4, 3, 0),
					(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 5, 4, 0),
					(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 6, 5, 0),
					(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 7, 6, 0),
					(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 8, 7, 0),
					(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 9, 8, 0),
					(" . PLUGIN_MASSOCSIMPORT_DETAIL . ", 5, 1, 0),
					(" . PLUGIN_MASSOCSIMPORT_DETAIL . ", 2, 2, 0),
					(" . PLUGIN_MASSOCSIMPORT_DETAIL . ", 3, 3, 0),
					(" . PLUGIN_MASSOCSIMPORT_DETAIL . ", 4, 4, 0),
					(" . PLUGIN_MASSOCSIMPORT_DETAIL . ", 6, 5, 0);";
	$DB->query($query) or die($DB->error());

	}

	return true;
}

function plugin_massocsimport_needUpgrade() {
	if (TableExists("glpi_plugin_mass_ocs_import"))
		return true;
	else 
		return false;
}

function plugin_massocsimport_upgrade11to12() {
	global $DB;

	// This index is provided in GLPI 0.71, not in 0.70, so we create it
	if (!isIndex("glpi_ocs_link", "last_ocs_update")) {
		$query = "ALTER TABLE `glpi_ocs_link` ADD INDEX `last_ocs_update` ( `ocs_server_id` , `last_ocs_update` )";
		$DB->query($query) or die($DB->error());
	}

	// plugin tables
	if (!TableExists("glpi_plugin_mass_ocs_import_config")) {
		$query = "CREATE TABLE `glpi_plugin_mass_ocs_import_config` (
													  		 `ID` int(11) NOT NULL,
													  		 `enable_logging` int(1) NOT NULL default '1',
													  		 `thread_log_frequency` int(4) NOT NULL default '10',
													  		 `display_empty` int(1) NOT NULL default '1',
													  		 `delete_frequency` int(4) NOT NULL default '0',
													  		 `import_limit` int(11) NOT NULL default '0',
													  		 `default_ocs_server` int(11) NOT NULL default '-1',
														     `delay_refresh` varchar(4) NOT NULL default '0',
														     `delete_empty_frequency` int(4) NOT NULL default '0',
													  		 PRIMARY KEY  (`ID`)
														) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci ";

		$DB->query($query) or die($DB->error());

		$query = "INSERT INTO `glpi_plugin_mass_ocs_import_config` (`ID`, `enable_logging`, `thread_log_frequency`, `display_empty`, `delete_frequency`, `delete_empty_frequency`, `import_limit`,`default_ocs_server` ) " .
		" VALUES (1, 1, 5, 1, 2, 2, 0,-1);";

		$DB->query($query) or die($DB->error());
	}

	if (!FieldExists("glpi_plugin_mass_ocs_import_config", "warn_if_not_imported")) {
		$query = "ALTER TABLE `glpi_plugin_mass_ocs_import_config` ADD `warn_if_not_imported` int(1) NOT NULL default '0';";
		$DB->query($query) or die($DB->error());
	}

	if (!FieldExists("glpi_plugin_mass_ocs_import_config", "not_imported_threshold")) {
		$query = "ALTER TABLE `glpi_plugin_mass_ocs_import_config` ADD `not_imported_threshold` int(11) NOT NULL default '0';";
		$DB->query($query) or die($DB->error());
	}
}
function plugin_massocsimport_upgrade121to13() {
	global $DB;
	if (TableExists("glpi_plugin_mass_ocs_import_config")) {
		$tables = array (
			"glpi_plugin_massocsimport_servers" => "glpi_plugin_mass_ocs_import_servers",
			"glpi_plugin_massocsimport" => "glpi_plugin_mass_ocs_import",
			"glpi_plugin_massocsimport_config" => "glpi_plugin_mass_ocs_import_config",
			"glpi_plugin_massocsimport_not_imported" => "glpi_plugin_mass_ocs_import_not_imported"
		);

		foreach ($tables as $new => $old) {
			if (!TableExists($new)) {
				$query = "RENAME TABLE `$old` TO `$new`;";
				$DB->query($query) or die($DB->error());
			}
		}

		$query = "ALTER TABLE `glpi_plugin_massocsimport` CHANGE `process_id` `process_id` BIGINT( 20 ) NOT NULL DEFAULT '0'";
		$DB->query($query) or die($DB->error());

		
		if (!FieldExists("glpi_plugin_massocsimport_config","comments")) {
			$query = " ALTER TABLE `glpi_plugin_massocsimport_config` ADD `comments` TEXT NULL  ";
			$DB->query($query) or die($DB->error());
		}

		if (!FieldExists("glpi_plugin_massocsimport", "noupdate_machines_number")) {
			$query = " ALTER TABLE `glpi_plugin_massocsimport` ADD `noupdate_machines_number` int(11) NOT NULL default '0'";
			$DB->query($query) or die($DB->error());
		}

		if (!TableExists("glpi_plugin_massocsimport_details")) {
			$query = "CREATE TABLE IF NOT EXISTS `glpi_plugin_massocsimport_details` (
						  `ID` int(11) NOT NULL auto_increment,
						  `process_id` bigint(10) NOT NULL default '0',
						  `thread_id` int(4) NOT NULL default '0',
						  `ocs_id` int(11) NOT NULL default '0',
						  `glpi_id` int(11) NOT NULL default '0',
						  `action` int(11) NOT NULL default '0',
						  `process_time` datetime DEFAULT NULL,
						  `ocs_server_id` int(4) NOT NULL default '1',
						  PRIMARY KEY  (`ID`),
						  KEY `end_time` (`process_time`)
						) ENGINE=MyISAM  DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci";
			$DB->query($query) or die($DB->error());
		}

		//Add fields to the default view		
		$query = "INSERT INTO `glpi_display` (`type`, `num`, `rank`, `FK_users`) VALUES
						(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 2, 1, 0),
						(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 3, 2, 0),
						(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 4, 3, 0),
						(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 5, 4, 0),
						(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 6, 5, 0),
						(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 7, 6, 0),
						(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 8, 7, 0),
						(" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . ", 9, 8, 0),
						(" . PLUGIN_MASSOCSIMPORT_DETAIL . ", 5, 1, 0),
						(" . PLUGIN_MASSOCSIMPORT_DETAIL . ", 2, 2, 0),
						(" . PLUGIN_MASSOCSIMPORT_DETAIL . ", 3, 3, 0),
						(" . PLUGIN_MASSOCSIMPORT_DETAIL . ", 4, 4, 0),
						(" . PLUGIN_MASSOCSIMPORT_DETAIL . ", 6, 5, 0);";
		$DB->query($query) or die($DB->error());

		$drop_fields = array (
			//Was not used, debug only...
			"glpi_plugin_massocsimport_config" => "warn_if_not_imported",
			"glpi_plugin_massocsimport_config" => "not_imported_threshold",
			//Logging must always be enable !
			"glpi_plugin_massocsimport_config" => "enable_logging",
			"glpi_plugin_massocsimport_config" => "delete_empty_frequency"
		);
		foreach ($drop_fields as $table => $field) {
			if (FieldExists($table, $field)) {
				$query = "ALTER TABLE `$table` DROP `$field`";
				$DB->query($query) or die($DB->error());
			}
		}
	}
}

function plugin_massocsimport_uninstall() {
	$DB = new DB;

	$tables = array (
		"glpi_plugin_massocsimport",
		"glpi_plugin_massocsimport_servers",
		"glpi_plugin_massocsimport_config",
		"glpi_plugin_massocsimport_not_imported",
		"glpi_plugin_massocsimport_details"
	);

	foreach ($tables as $table) {
		$query = "DROP TABLE  IF EXISTS `$table`;";
		$DB->query($query) or die($DB->error());
	}

	$query = "DELETE FROM glpi_display WHERE type IN (" . PLUGIN_MASSOCSIMPORT_NOTIMPORTED . "," . PLUGIN_MASSOCSIMPORT_DETAIL . ")";
	$DB->query($query) or die($DB->error());
}

function showOcsReportsConsole($ID) {
	global $LANG;

	$ocsconfig = getOcsConf($ID);

	echo "<div align='center'>";
	if ($ocsconfig["ocs_url"] != '')
		echo "<iframe src='" . $ocsconfig["ocs_url"] . "/index.php?multi=4' width='95%' height='650' >";
	echo "</div>";
}
?>
