<?php


/*
 ----------------------------------------------------------------------
 GLPI - Gestionnaire Libre de Parc Informatique
 Copyright (C) 2003-2008 by the INDEPNET Development Team.

 http://indepnet.net/   http://glpi-project.org/
 ----------------------------------------------------------------------

 LICENSE

	This file is part of GLPI.

    GLPI is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    GLPI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GLPI; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ------------------------------------------------------------------------
*/

// Original Author of file: Walid Nouh
// Purpose of file:
// ----------------------------------------------------------------------
function plugin_massocsimport_getProcessStatus($pid) {
	global $DB;
	$sql = "SELECT status FROM `glpi_plugin_massocsimport` WHERE process_id=" . $pid;
	$result = $DB->query($sql);
	$status = 0;
	$thread_number = 0;

	$thread_number = $DB->numrows($result);

	while ($thread = $DB->fetch_array($result)) {
		$status += $thread["status"];
	}

	if ($status < $thread_number * PLUGIN_MASSOCSIMPORT_STATE_FINISHED)
		return PLUGIN_MASSOCSIMPORT_STATE_RUNNING;
	else
		return PLUGIN_MASSOCSIMPORT_STATE_FINISHED;
}

function plugin_massocsimport_deleteOldProcesses($delete_frequency)
{
	global $DB;
	
	$nbdel=0;

	if ($delete_frequency > 0)
	{
		$sql = "DELETE FROM `glpi_plugin_massocsimport` WHERE status=" . PLUGIN_MASSOCSIMPORT_STATE_FINISHED . " AND end_time < DATE_ADD(NOW(), INTERVAL -".$delete_frequency." HOUR)";
		$result = $DB->query($sql);
		if ($result) $nbdel+=$DB->affected_rows();
	}

	return $nbdel;
}

function plugin_massocsimport_getOcsServerIdsAndNames($ocs_servers)
{
	$servers = array();
	if (is_array($ocs_servers))
	{
		foreach ($servers as $server)
		{
			$ocsConfig = getOcsConf($server);
			$servers[$ocsConfig["ID"]] = $servers[$ocsConfig["name"]];
		}		 
	}
	else
	{
		$ocsConfig = getOcsConf($ocs_servers);
		$servers[$ocsConfig["ID"]] = $servers[$ocsConfig["name"]];
	}
	
	return $servers;
}

function cronDeleteThreads()
{
	$config = new MassOCSImportConfig;
	$config->getFromDB(1);
	return plugin_massocsimport_deleteOldProcesses($config->fields["delete_frequency"]);
}

function plugin_massocsimport_showProcesses($target)
 {
  		global $DB,$LANG,$CFG_GLPI,$LANG;
  		
  		$canedit= haveRight("ocsng","w");
  		
  		$config = new MassOCSImportConfig;
  		$config->getFromDB(1);

		//First of all, deleted old processes
		plugin_massocsimport_deleteOldProcesses($config->fields["delete_frequency"]);
		
		if ($config->fields["delete_frequency"]>0)  
			$minfreq=$config->fields["delete_frequency"];
		else
			$minfreq = 9999;

  		$imported_number     = new MiniStat;
  		$synchronized_number = new MiniStat;
  		$linked_number       = new MiniStat;
  		$failed_number       = new MiniStat;
  		$notupdated_number   = new MiniStat;
  		$process_time        = new MiniStat;
  		
		$sql = 	"SELECT ID, process_id, SUM(total_number_machines) AS total_machines,ocs_server_id, status, COUNT(*) AS threads_number, " .
			"MIN(start_time) AS starting_date, MAX(end_time) AS ending_date, TIME_TO_SEC(MAX(end_time))-TIME_TO_SEC(MIN(start_time)) AS duree, " .
		     "SUM(imported_machines_number) AS imported_machines, SUM(synchronized_machines_number) AS synchronized_machines, " .
			"SUM(linked_machines_number) AS linked_machines, SUM(failed_rules_machines_number) AS failed_rules_machines, " .
			"SUM(noupdate_machines_number) AS noupdate_machines, " .
			"end_time >= DATE_ADD(NOW(), INTERVAL -" . $minfreq . " HOUR) AS DoStat ".
		      	"FROM `glpi_plugin_massocsimport` GROUP BY process_id ORDER BY ID DESC";
	     	$result = $DB->query($sql);
      
      	echo "<div align='center'>";
		
		echo "<form name='processes' id='processes' action=\"".$target."\" method=\"post\">";

		echo "<table class='tab_cadre' cellpadding='9' width='80%'>";
		echo "<tr><th colspan='14'>" . $LANG["massocsimport"]["common"][1] . "</th></tr>";
		echo "<tr>"; 
		echo"<th>&nbsp;</th>";
		echo"<th>&nbsp;</th>";
		echo"<th>".$LANG["massocsimport"]["common"][10]."</th>";
		echo"<th>".$LANG["massocsimport"]["common"][4]."</th>";
		echo"<th>".$LANG["massocsimport"]["common"][2]."</th>";
		echo"<th>".$LANG["massocsimport"]["common"][3]."</th>";
		echo"<th>".$LANG["massocsimport"]["common"][5]."</th>";
		echo"<th>".$LANG["massocsimport"]["common"][6]."</th>";
		echo"<th>".$LANG["massocsimport"]["common"][19]."</th>";
		echo"<th>".$LANG["massocsimport"]["common"][18]."</th>";
		echo"<th>".$LANG["massocsimport"]["common"][20]."</th>";		
		echo"<th>".$LANG["massocsimport"]["common"][7]."</th>";		
		echo"<th>".$LANG["massocsimport"]["common"][11]."</th>";
		echo"<th>&nbsp;</th>";
		echo "</th></tr>\n";

		if ($DB->numrows($result)) {
		
			while ($thread = $DB->fetch_array($result)){
				if ($config->fields["display_empty"] || $thread["status"] != PLUGIN_MASSOCSIMPORT_STATE_FINISHED || (!$config->fields["display_empty"] && $thread["total_machines"] > 0 && $thread["status"] == PLUGIN_MASSOCSIMPORT_STATE_FINISHED))
				{
					if ($thread["DoStat"] && $thread["status"] == PLUGIN_MASSOCSIMPORT_STATE_FINISHED) {
						$imported_number->AddValue($thread["imported_machines"]);
						$synchronized_number->AddValue($thread["synchronized_machines"]);
						$linked_number->AddValue($thread["linked_machines"]);
						$failed_number->AddValue($thread["failed_rules_machines"]);
						$notupdated_number->AddValue($thread["noupdate_machines"]);						
						$process_time->AddValue($thread["duree"]);
					}
					else if ($imported_number->GetCount()>0) {
						plugin_massocsimport_showshowStat($minfreq, $imported_number, $synchronized_number, $linked_number, $failed_number, $notupdated_number, $process_time);
						$imported_number->Reset();
					}
					echo "<tr class='tab_bg_1'>"; 
					
					if ($canedit){
						echo "<td width='10'>";
						echo "<input type='checkbox' name='item[".$thread["process_id"]."]' value='1'>";
						echo "</td>";
					}
					else
						 echo "<td width='10'>&nbsp;</td>";
				
					echo "<td align='center'><a href=\"./front/plugin_massocsimport.process.form.php?pid=".$thread["process_id"]."\">".$thread["process_id"]."</a></td>";
					echo "<td align='center'>";
					
					plugin_massocsimport_displayProcessStatusIcon(
						plugin_massocsimport_getProcessStatus($thread["process_id"])
					);
						
					echo "</td>";
					echo "<td align='center'>".$thread["threads_number"]."</td>";
					echo "<td align='center'>".convDateTime($thread["starting_date"])."</td>";
					echo "<td align='center'>".convDateTime($thread["ending_date"])."</td>";
					echo "<td align='center'>".$thread["imported_machines"]."</td>";
					echo "<td align='center'>".$thread["synchronized_machines"]."</td>";
					echo "<td align='center'>".$thread["linked_machines"]."</td>";
					echo "<td align='center'>".$thread["failed_rules_machines"]."</td>";
					echo "<td align='center'>".$thread["noupdate_machines"]."</td>";
					
					echo "<td align='center'>";
					if ($thread["status"] == PLUGIN_MASSOCSIMPORT_STATE_FINISHED)
						echo timestampToString($thread["duree"]);
					else
						echo "-----";	
					
					echo "</td>";
					
					echo "<td align='center'>"; 
					if ($thread["ocs_server_id"] != -1)
					{
						$ocsConfig = getOcsConf($thread["ocs_server_id"]);
						echo "<a href=\"".GLPI_ROOT."/front/ocsng.form.php?ID=".$ocsConfig["ID"]."\">".$ocsConfig["name"]."</a>";
					}
					else
						echo $LANG["massocsimport"]["config"][22];
						
					echo "</td>";
					echo "<td align='center'><a href=\"front/plugin_massocsimport.details.php?reset=reset_before&field[0]=5&contains[0]=".$thread["process_id"]."\"><img  src='".$CFG_GLPI["root_doc"]."/pics/rdv.png'</a></td>";
			
					echo "</tr>\n";
				}	
			}
		}

		if ($imported_number->GetCount()>0) {
			plugin_massocsimport_showshowStat($minfreq, $imported_number, $synchronized_number, $linked_number, $failed_number, $notupdated_number, $process_time);
		}
		echo "</table>";
		
		if ($canedit){
			echo "<div align='center'>";
			echo "<table width='80%'>";
			echo "<tr><td><img src=\"".$CFG_GLPI["root_doc"]."/pics/arrow-left.png\" alt=''></td>"; 
			echo "<td align='center'><a onclick= \"if ( markCheckboxes('processes') ) return false;\" href='#'>".$LANG["buttons"][18]."</a></td>";
	
			echo "<td>/</td><td align='center'><a onclick= \"if ( unMarkCheckboxes('processes') ) return false;\" href='#'>".$LANG["buttons"][19]."</a>";
			echo "</td><td align='left' width='80%'>";
			echo "<input type='submit' name='delete_processes' value=\"".$LANG["buttons"][6]."\" class='submit'>";
			echo "</td>";
			echo "</table>";
	
			echo "</div>";
		}		
		
}

 function plugin_massocsimport_showshowStat($duree, &$imported, &$synchronized, &$linked, &$failed, &$notupdated, &$time)
  {
	global $LANG;

	echo "<tr><th colspan='14'>" . $LANG["massocsimport"]["common"][30];
	if ($duree < 9999) echo " (" . $duree . " " . $LANG["massocsimport"]["time"][1] . ")";
	echo "</th></tr>";

	echo "<tr class='tab_bg_1'>";
	echo "<td align='right' colspan='6'>" . $LANG["massocsimport"]["common"][25] . "<br />" . $LANG["massocsimport"]["common"][26] . "<br />" . $LANG["massocsimport"]["common"][27] . "<br />" . $LANG["massocsimport"]["common"][28] . "</td>";
	echo "<td align='center'>" . $imported->GetMinimum() . "<br />" . $imported->GetMaximum() . "<br />" . round($imported->GetAverage(),2) . "<br />" . $imported->GetTotal() . "</td>";
	echo "<td align='center'>" . $synchronized->GetMinimum() . "<br />" . $synchronized->GetMaximum() . "<br />" . round($synchronized->GetAverage(),2) . "<br />" . $synchronized->GetTotal() . "</td>";
	echo "<td align='center'>" . $linked->GetMinimum() . "<br />" . $linked->GetMaximum() . "<br />" . round($linked->GetAverage(),2) . "<br />" . $linked->GetTotal() . "</td>";
	echo "<td align='center'>" . $failed->GetMinimum() . "<br />" . $failed->GetMaximum() . "<br />" . round($failed->GetAverage(),2) . "<br />&nbsp;</td>";
	echo "<td align='center'>" . $notupdated->GetMinimum() . "<br />" . $notupdated->GetMaximum() . "<br />" . round($notupdated->GetAverage(),2) . "<br />&nbsp;</td>";
	echo "<td align='center'>" . timestampToString($time->GetMinimum()) . "<br />" . timestampToString($time->GetMaximum()) . "<br />" . 
		timestampToString(round($time->GetAverage())) . "<br />" . timestampToString($time->GetTotal()) . "</td>";
	if ($time->GetTotal()>0) {
		echo "<td align='center' colspan='2'>" . $LANG["massocsimport"]["common"][29] . "<br />" . 
		round(($imported->GetTotal()+$synchronized->GetTotal()+$linked->GetTotal()+$failed->GetTotal())/$time->GetTotal(),2) . " pc/s</td>";
	}
	else	echo "<td>&nbsp;</td><td>&nbsp;</td>";
	echo "</tr>\n";
	echo "<tr><th colspan='14'>-----</th></tr>\n";
   }

   
  function plugin_massocsimport_showErrorLog (){
	global $CFG_GLPI, $DB;

	$fic=GLPI_LOG_DIR."/ocsng_fullsync.log";
	if (!is_file($fic)) return false;
	
	$size=filesize($fic);

	if ($size>20000) {
		$logfile=file_get_contents($fic,0,NULL,$size-20000,20000);
		$events = explode("\n", $logfile);
		// Remove fist partial event
		array_shift($events);
	} else {
		$logfile=file_get_contents($fic);
		$events = explode("\n\n", $logfile);
	}

	// Remove last empty event
	array_pop($events);
	
	$number = count($events); 

	$SEARCH_OPTION=getSearchOptions();
	
	if (isset($_REQUEST["start"])) {
		$start = $_REQUEST["start"];
	} else {
		$start = 0;
	}

	if ($number < 1) return plugin_massocsimport_lognothing();
	
	if ($start>$number) $start=$number;

	// Display the pager
	printAjaxPager("Logfile : ocsng_fullsync.log",$start,$number);
	
	// Output events
	echo "<div class='center'><table class='tab_cadre_fixe'>";
		
	echo "<tr><th>Message</th></tr>";
	
	for ($i=$start ; $i<($start+$_SESSION['glpilist_limit']) && $i<count($events) ; $i++) {
		$lines = explode ("\n",$events[$i]);
		echo "<tr class='tab_bg_2' valign='top'><td>".$lines[0]."</td>";
		echo "</tr>";
	}
	echo "</table></div>";
}
   
  function plugin_massocsimport_lognothing (){
	global $LANG;
	
	echo "<div class='center'>";
	echo "<table class='tab_cadre_fixe'>";
	echo "<tr><th>No record found</th></tr>";
	echo "</table>";
	echo "</div><br>";
	
	return false;
}
   
  function plugin_massocsimport_displayProcessStatusIcon($status)
  {
 				switch ($status) {
					case PLUGIN_MASSOCSIMPORT_STATE_FINISHED :
						echo "<img src='".GLPI_ROOT."/plugins/massocsimport/pics/export.png'>";
						break;
					case PLUGIN_MASSOCSIMPORT_STATE_RUNNING :
						echo "<img src='".GLPI_ROOT."/plugins/massocsimport/pics/wait.png'>";
						$finished = false;
						break;
					case PLUGIN_MASSOCSIMPORT_STATE_STARTED :
						echo "<img src='".GLPI_ROOT."/plugins/massocsimport/pics/ok2.png'>";
						$finished = false;
						break;
				}
  } 
?>